#include "zhash.h"
#include <vector>
#include <fstream>
#include <string>
#include <algorithm>

int main() {
	std::size_t resolution = 256;
	std::size_t texture_count = 64;
	ZHashSampler sampler(resolution, resolution);
	// Prepare the output array. We store two 2D samples in one RGBA value.
	std::size_t sample_pitch = 2;
	std::size_t x_pitch = 2 * sample_pitch;
	std::size_t y_pitch = resolution * x_pitch;
	std::size_t texture_pitch = resolution * y_pitch;
	std::vector<double> samples(texture_count * texture_pitch);
	// Generate the samples
	for (std::size_t texture = 0; texture != texture_count; ++texture)
		for (std::size_t y = 0; y != resolution; ++y)
			for (std::size_t x = 0; x != resolution; ++x)
				for (std::size_t sample = 0; sample != 2; ++sample)
				// Repurpose dimension indices for sample indices. The reason
				// is that we mostly care about one sample per pixel per frame
				// and want different textures to be independent.
				sampler.Z2D(static_cast<int>(x), static_cast<int>(y), 1,
					&samples[texture * texture_pitch + y * y_pitch + x * x_pitch + sample * sample_pitch],
					static_cast<int>(texture * 2 + sample));
	// Store samples as 16-bit fixed point
	std::ofstream output_file("../../vulkan_renderer/data/noise/ahmed_2d_rgba_" + std::to_string(resolution) + "x" + std::to_string(resolution) + "_" + std::to_string(texture_count) + ".blob", std::ios_base::binary);
	for (double value : samples) {
		// Samples with value exactly 1 get clamped to the next smaller number
		uint16_t quantized = static_cast<uint16_t>(std::max(0.0, std::min(65534.0, 65536.0 * value)));
		output_file.write(reinterpret_cast<char*>(&quantized), sizeof(quantized));
	}
}
