/*
 * Define a sampler that uses scrambled Z-ordering to distribute Sobol indexes.
 * 2020-12-30: Modified to create a lookup table independent of PBRT.
 * 2020-05-20: Created by Abdalla Ahmed from an earlier version.
 * 2020-07-09: Adapted by Abdalla Ahmed to use a hash function.
 * 2020-08-17: Revised by Abdalla Ahmed for final submission with the paper.
 */
#pragma once

#include <cstdint>
#include "zcommon.h"

// ZHashSampler Declarations
class ZHashSampler {
public:
	// All parameters must be powers of two
	ZHashSampler(
		int64_t width, int64_t height
	);

	void Z2D(
		int x, int y,
		int nSamples,
		double* samples,
		int dim
	);
	TileInfo getChildInfo(
		uint32_t X, uint32_t Y,												// Absolute coordinates of child tile.
		TileInfo rootInfo,													// ID and seqNo of root tile
		int digits,															// Subdivision depth of parent tile; only these digits are considered!
		int dim																// Array of bytes indicating the ranks of children for each ID
	);
	uint32_t seqNo2position(
		uint32_t seqNo,
		uint32_t tileID,													// ID of root node
		int bits,															// depth in bits rather than base-4 digits, to handle odd powers of two
		int dim																// Table of permutations (per node ID)
	);

private:
	int resolution, log2Resolution;
	static const int DIMENSIONS = 1024;
};
