//  Copyright (C) 2021, Christoph Peters, Karlsruhe Institute of Technology
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <https://www.gnu.org/licenses/>.


// This file is automatically generated. It implements GLM-like functionality
// but is less complete. The benefit is that it is a painless dependency.
#pragma once

struct vec2 {
	inline Float& operator[](unsigned int entry_index) {
		return *((&x) + entry_index);
	}
	inline Float operator[](unsigned int entry_index) const {
		return *((&x) + entry_index);
	}

	union {
		struct { Float x, y; };
	};
};

struct vec3 {
	inline Float& operator[](unsigned int entry_index) {
		return *((&x) + entry_index);
	}
	inline Float operator[](unsigned int entry_index) const {
		return *((&x) + entry_index);
	}

	union {
		struct { Float x, y, z; };
		struct { Float x_1; vec2 yz; };
		struct { vec2 xy; Float z_1; };
	};
};

struct vec4 {
	inline Float& operator[](unsigned int entry_index) {
		return *((&x) + entry_index);
	}
	inline Float operator[](unsigned int entry_index) const {
		return *((&x) + entry_index);
	}

	union {
		struct { Float x, y, z, w; };
		struct { Float x_1; vec3 yzw; };
		struct { vec2 xy; vec2 zw; };
		struct { vec3 xyz; Float w_1; };
		struct { Float x_3; vec2 yz; Float w_2; };
	};
};

struct vec5 {
	inline Float& operator[](unsigned int entry_index) {
		return *((&x) + entry_index);
	}
	inline Float operator[](unsigned int entry_index) const {
		return *((&x) + entry_index);
	}

	union {
		struct { Float x, y, z, w, a; };
		struct { Float x_1; vec4 yzwa; };
		struct { vec2 xy; vec3 zwa; };
		struct { vec3 xyz; vec2 wa; };
		struct { vec4 xyzw; Float a_1; };
		struct { Float x_3; vec2 yz; vec2 wa_1; };
		struct { Float x_4; vec3 yzw; Float a_2; };
		struct { vec2 xy_2; vec2 zw; Float a_3; };
	};
};

struct vec6 {
	inline Float& operator[](unsigned int entry_index) {
		return *((&x) + entry_index);
	}
	inline Float operator[](unsigned int entry_index) const {
		return *((&x) + entry_index);
	}

	union {
		struct { Float x, y, z, w, a, b; };
		struct { Float x_1; vec5 yzwab; };
		struct { vec2 xy; vec4 zwab; };
		struct { vec3 xyz; vec3 wab; };
		struct { vec4 xyzw; vec2 ab; };
		struct { vec5 xyzwa; Float b_1; };
		struct { Float x_3; vec2 yz; vec3 wab_1; };
		struct { Float x_4; vec3 yzw; vec2 ab_1; };
		struct { Float x_5; vec4 yzwa; Float b_2; };
		struct { vec2 xy_2; vec2 zw; vec2 ab_2; };
		struct { vec2 xy_3; vec3 zwa; Float b_3; };
		struct { vec3 xyz_2; vec2 wa; Float b_4; };
	};
};

struct mat2 {
	inline mat2() {}
	inline mat2(Float diagonal) {
		columns[0][0] = diagonal;
		columns[0][1] = 0;
		columns[1][0] = 0;
		columns[1][1] = diagonal;
	}
	inline mat2(Float col_0_row_0, Float col_0_row_1, Float col_1_row_0, Float col_1_row_1) {
		columns[0][0] = col_0_row_0;
		columns[0][1] = col_0_row_1;
		columns[1][0] = col_1_row_0;
		columns[1][1] = col_1_row_1;
	}
	inline mat2(vec2 col_0, vec2 col_1) {
		columns[0] = col_0;
		columns[1] = col_1;
	}
	inline vec2& operator[](unsigned int col) {
		return columns[col];
	}
	inline const vec2& operator[](unsigned int col) const{
		return columns[col];
	}

	vec2 columns[2];
};

struct mat3 {
	inline mat3() {}
	inline mat3(Float diagonal) {
		columns[0][0] = diagonal;
		columns[0][1] = 0;
		columns[0][2] = 0;
		columns[1][0] = 0;
		columns[1][1] = diagonal;
		columns[1][2] = 0;
		columns[2][0] = 0;
		columns[2][1] = 0;
		columns[2][2] = diagonal;
	}
	inline mat3(Float col_0_row_0, Float col_0_row_1, Float col_0_row_2, Float col_1_row_0, Float col_1_row_1, Float col_1_row_2, Float col_2_row_0, Float col_2_row_1, Float col_2_row_2) {
		columns[0][0] = col_0_row_0;
		columns[0][1] = col_0_row_1;
		columns[0][2] = col_0_row_2;
		columns[1][0] = col_1_row_0;
		columns[1][1] = col_1_row_1;
		columns[1][2] = col_1_row_2;
		columns[2][0] = col_2_row_0;
		columns[2][1] = col_2_row_1;
		columns[2][2] = col_2_row_2;
	}
	inline mat3(vec3 col_0, vec3 col_1, vec3 col_2) {
		columns[0] = col_0;
		columns[1] = col_1;
		columns[2] = col_2;
	}
	inline vec3& operator[](unsigned int col) {
		return columns[col];
	}
	inline const vec3& operator[](unsigned int col) const{
		return columns[col];
	}

	vec3 columns[3];
};

inline vec2 make_vec2(Float scalar) {
	vec2 result;
	result.x = scalar;
	result.y = scalar;
	return result;
}
inline vec2 make_vec2(Float x, Float y) {
	vec2 result;
	result.x = x;
	result.y = y;
	return result;
}

inline vec2 operator-(const vec2& rhs) {
	vec2 result;
	result.x = -rhs.x;
	result.y = -rhs.y;
	return result;
}

inline vec2 operator+(Float lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs + rhs.x;
	result.y = lhs + rhs.y;
	return result;
}

inline vec2 operator+(const vec2& lhs, Float rhs) {
	vec2 result;
	result.x = lhs.x + rhs;
	result.y = lhs.y + rhs;
	return result;
}

inline vec2 operator+(const vec2& lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs.x + rhs.x;
	result.y = lhs.y + rhs.y;
	return result;
}

inline const vec2& operator+=(vec2& lhs, Float rhs) {
	lhs.x += rhs;
	lhs.y += rhs;
	return lhs;
}

inline const vec2& operator+=(vec2& lhs, const vec2& rhs) {
	lhs.x += rhs.x;
	lhs.y += rhs.y;
	return lhs;
}

inline vec2 operator-(Float lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs - rhs.x;
	result.y = lhs - rhs.y;
	return result;
}

inline vec2 operator-(const vec2& lhs, Float rhs) {
	vec2 result;
	result.x = lhs.x - rhs;
	result.y = lhs.y - rhs;
	return result;
}

inline vec2 operator-(const vec2& lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs.x - rhs.x;
	result.y = lhs.y - rhs.y;
	return result;
}

inline const vec2& operator-=(vec2& lhs, Float rhs) {
	lhs.x -= rhs;
	lhs.y -= rhs;
	return lhs;
}

inline const vec2& operator-=(vec2& lhs, const vec2& rhs) {
	lhs.x -= rhs.x;
	lhs.y -= rhs.y;
	return lhs;
}

inline vec2 operator*(Float lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs * rhs.x;
	result.y = lhs * rhs.y;
	return result;
}

inline vec2 operator*(const vec2& lhs, Float rhs) {
	vec2 result;
	result.x = lhs.x * rhs;
	result.y = lhs.y * rhs;
	return result;
}

inline vec2 operator*(const vec2& lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs.x * rhs.x;
	result.y = lhs.y * rhs.y;
	return result;
}

inline const vec2& operator*=(vec2& lhs, Float rhs) {
	lhs.x *= rhs;
	lhs.y *= rhs;
	return lhs;
}

inline const vec2& operator*=(vec2& lhs, const vec2& rhs) {
	lhs.x *= rhs.x;
	lhs.y *= rhs.y;
	return lhs;
}

inline vec2 operator/(Float lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs / rhs.x;
	result.y = lhs / rhs.y;
	return result;
}

inline vec2 operator/(const vec2& lhs, Float rhs) {
	vec2 result;
	result.x = lhs.x / rhs;
	result.y = lhs.y / rhs;
	return result;
}

inline vec2 operator/(const vec2& lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs.x / rhs.x;
	result.y = lhs.y / rhs.y;
	return result;
}

inline const vec2& operator/=(vec2& lhs, Float rhs) {
	lhs.x /= rhs;
	lhs.y /= rhs;
	return lhs;
}

inline const vec2& operator/=(vec2& lhs, const vec2& rhs) {
	lhs.x /= rhs.x;
	lhs.y /= rhs.y;
	return lhs;
}

inline vec3 make_vec3(Float scalar) {
	vec3 result;
	result.x = scalar;
	result.y = scalar;
	result.z = scalar;
	return result;
}
inline vec3 make_vec3(Float x, vec2 yz) {
	vec3 result;
	result.x = x;
	result.yz = yz;
	return result;
}
inline vec3 make_vec3(vec2 xy, Float z) {
	vec3 result;
	result.xy = xy;
	result.z = z;
	return result;
}
inline vec3 make_vec3(Float x, Float y, Float z) {
	vec3 result;
	result.x = x;
	result.y = y;
	result.z = z;
	return result;
}

inline vec3 operator-(const vec3& rhs) {
	vec3 result;
	result.x = -rhs.x;
	result.y = -rhs.y;
	result.z = -rhs.z;
	return result;
}

inline vec3 operator+(Float lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs + rhs.x;
	result.y = lhs + rhs.y;
	result.z = lhs + rhs.z;
	return result;
}

inline vec3 operator+(const vec3& lhs, Float rhs) {
	vec3 result;
	result.x = lhs.x + rhs;
	result.y = lhs.y + rhs;
	result.z = lhs.z + rhs;
	return result;
}

inline vec3 operator+(const vec3& lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs.x + rhs.x;
	result.y = lhs.y + rhs.y;
	result.z = lhs.z + rhs.z;
	return result;
}

inline const vec3& operator+=(vec3& lhs, Float rhs) {
	lhs.x += rhs;
	lhs.y += rhs;
	lhs.z += rhs;
	return lhs;
}

inline const vec3& operator+=(vec3& lhs, const vec3& rhs) {
	lhs.x += rhs.x;
	lhs.y += rhs.y;
	lhs.z += rhs.z;
	return lhs;
}

inline vec3 operator-(Float lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs - rhs.x;
	result.y = lhs - rhs.y;
	result.z = lhs - rhs.z;
	return result;
}

inline vec3 operator-(const vec3& lhs, Float rhs) {
	vec3 result;
	result.x = lhs.x - rhs;
	result.y = lhs.y - rhs;
	result.z = lhs.z - rhs;
	return result;
}

inline vec3 operator-(const vec3& lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs.x - rhs.x;
	result.y = lhs.y - rhs.y;
	result.z = lhs.z - rhs.z;
	return result;
}

inline const vec3& operator-=(vec3& lhs, Float rhs) {
	lhs.x -= rhs;
	lhs.y -= rhs;
	lhs.z -= rhs;
	return lhs;
}

inline const vec3& operator-=(vec3& lhs, const vec3& rhs) {
	lhs.x -= rhs.x;
	lhs.y -= rhs.y;
	lhs.z -= rhs.z;
	return lhs;
}

inline vec3 operator*(Float lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs * rhs.x;
	result.y = lhs * rhs.y;
	result.z = lhs * rhs.z;
	return result;
}

inline vec3 operator*(const vec3& lhs, Float rhs) {
	vec3 result;
	result.x = lhs.x * rhs;
	result.y = lhs.y * rhs;
	result.z = lhs.z * rhs;
	return result;
}

inline vec3 operator*(const vec3& lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs.x * rhs.x;
	result.y = lhs.y * rhs.y;
	result.z = lhs.z * rhs.z;
	return result;
}

inline const vec3& operator*=(vec3& lhs, Float rhs) {
	lhs.x *= rhs;
	lhs.y *= rhs;
	lhs.z *= rhs;
	return lhs;
}

inline const vec3& operator*=(vec3& lhs, const vec3& rhs) {
	lhs.x *= rhs.x;
	lhs.y *= rhs.y;
	lhs.z *= rhs.z;
	return lhs;
}

inline vec3 operator/(Float lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs / rhs.x;
	result.y = lhs / rhs.y;
	result.z = lhs / rhs.z;
	return result;
}

inline vec3 operator/(const vec3& lhs, Float rhs) {
	vec3 result;
	result.x = lhs.x / rhs;
	result.y = lhs.y / rhs;
	result.z = lhs.z / rhs;
	return result;
}

inline vec3 operator/(const vec3& lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs.x / rhs.x;
	result.y = lhs.y / rhs.y;
	result.z = lhs.z / rhs.z;
	return result;
}

inline const vec3& operator/=(vec3& lhs, Float rhs) {
	lhs.x /= rhs;
	lhs.y /= rhs;
	lhs.z /= rhs;
	return lhs;
}

inline const vec3& operator/=(vec3& lhs, const vec3& rhs) {
	lhs.x /= rhs.x;
	lhs.y /= rhs.y;
	lhs.z /= rhs.z;
	return lhs;
}

inline vec4 make_vec4(Float scalar) {
	vec4 result;
	result.x = scalar;
	result.y = scalar;
	result.z = scalar;
	result.w = scalar;
	return result;
}
inline vec4 make_vec4(Float x, vec3 yzw) {
	vec4 result;
	result.x = x;
	result.yzw = yzw;
	return result;
}
inline vec4 make_vec4(vec2 xy, vec2 zw) {
	vec4 result;
	result.xy = xy;
	result.zw = zw;
	return result;
}
inline vec4 make_vec4(vec3 xyz, Float w) {
	vec4 result;
	result.xyz = xyz;
	result.w = w;
	return result;
}
inline vec4 make_vec4(Float x, Float y, vec2 zw) {
	vec4 result;
	result.x = x;
	result.y = y;
	result.zw = zw;
	return result;
}
inline vec4 make_vec4(Float x, vec2 yz, Float w) {
	vec4 result;
	result.x = x;
	result.yz = yz;
	result.w = w;
	return result;
}
inline vec4 make_vec4(vec2 xy, Float z, Float w) {
	vec4 result;
	result.xy = xy;
	result.z = z;
	result.w = w;
	return result;
}
inline vec4 make_vec4(Float x, Float y, Float z, Float w) {
	vec4 result;
	result.x = x;
	result.y = y;
	result.z = z;
	result.w = w;
	return result;
}

inline vec4 operator-(const vec4& rhs) {
	vec4 result;
	result.x = -rhs.x;
	result.y = -rhs.y;
	result.z = -rhs.z;
	result.w = -rhs.w;
	return result;
}

inline vec4 operator+(Float lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs + rhs.x;
	result.y = lhs + rhs.y;
	result.z = lhs + rhs.z;
	result.w = lhs + rhs.w;
	return result;
}

inline vec4 operator+(const vec4& lhs, Float rhs) {
	vec4 result;
	result.x = lhs.x + rhs;
	result.y = lhs.y + rhs;
	result.z = lhs.z + rhs;
	result.w = lhs.w + rhs;
	return result;
}

inline vec4 operator+(const vec4& lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs.x + rhs.x;
	result.y = lhs.y + rhs.y;
	result.z = lhs.z + rhs.z;
	result.w = lhs.w + rhs.w;
	return result;
}

inline const vec4& operator+=(vec4& lhs, Float rhs) {
	lhs.x += rhs;
	lhs.y += rhs;
	lhs.z += rhs;
	lhs.w += rhs;
	return lhs;
}

inline const vec4& operator+=(vec4& lhs, const vec4& rhs) {
	lhs.x += rhs.x;
	lhs.y += rhs.y;
	lhs.z += rhs.z;
	lhs.w += rhs.w;
	return lhs;
}

inline vec4 operator-(Float lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs - rhs.x;
	result.y = lhs - rhs.y;
	result.z = lhs - rhs.z;
	result.w = lhs - rhs.w;
	return result;
}

inline vec4 operator-(const vec4& lhs, Float rhs) {
	vec4 result;
	result.x = lhs.x - rhs;
	result.y = lhs.y - rhs;
	result.z = lhs.z - rhs;
	result.w = lhs.w - rhs;
	return result;
}

inline vec4 operator-(const vec4& lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs.x - rhs.x;
	result.y = lhs.y - rhs.y;
	result.z = lhs.z - rhs.z;
	result.w = lhs.w - rhs.w;
	return result;
}

inline const vec4& operator-=(vec4& lhs, Float rhs) {
	lhs.x -= rhs;
	lhs.y -= rhs;
	lhs.z -= rhs;
	lhs.w -= rhs;
	return lhs;
}

inline const vec4& operator-=(vec4& lhs, const vec4& rhs) {
	lhs.x -= rhs.x;
	lhs.y -= rhs.y;
	lhs.z -= rhs.z;
	lhs.w -= rhs.w;
	return lhs;
}

inline vec4 operator*(Float lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs * rhs.x;
	result.y = lhs * rhs.y;
	result.z = lhs * rhs.z;
	result.w = lhs * rhs.w;
	return result;
}

inline vec4 operator*(const vec4& lhs, Float rhs) {
	vec4 result;
	result.x = lhs.x * rhs;
	result.y = lhs.y * rhs;
	result.z = lhs.z * rhs;
	result.w = lhs.w * rhs;
	return result;
}

inline vec4 operator*(const vec4& lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs.x * rhs.x;
	result.y = lhs.y * rhs.y;
	result.z = lhs.z * rhs.z;
	result.w = lhs.w * rhs.w;
	return result;
}

inline const vec4& operator*=(vec4& lhs, Float rhs) {
	lhs.x *= rhs;
	lhs.y *= rhs;
	lhs.z *= rhs;
	lhs.w *= rhs;
	return lhs;
}

inline const vec4& operator*=(vec4& lhs, const vec4& rhs) {
	lhs.x *= rhs.x;
	lhs.y *= rhs.y;
	lhs.z *= rhs.z;
	lhs.w *= rhs.w;
	return lhs;
}

inline vec4 operator/(Float lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs / rhs.x;
	result.y = lhs / rhs.y;
	result.z = lhs / rhs.z;
	result.w = lhs / rhs.w;
	return result;
}

inline vec4 operator/(const vec4& lhs, Float rhs) {
	vec4 result;
	result.x = lhs.x / rhs;
	result.y = lhs.y / rhs;
	result.z = lhs.z / rhs;
	result.w = lhs.w / rhs;
	return result;
}

inline vec4 operator/(const vec4& lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs.x / rhs.x;
	result.y = lhs.y / rhs.y;
	result.z = lhs.z / rhs.z;
	result.w = lhs.w / rhs.w;
	return result;
}

inline const vec4& operator/=(vec4& lhs, Float rhs) {
	lhs.x /= rhs;
	lhs.y /= rhs;
	lhs.z /= rhs;
	lhs.w /= rhs;
	return lhs;
}

inline const vec4& operator/=(vec4& lhs, const vec4& rhs) {
	lhs.x /= rhs.x;
	lhs.y /= rhs.y;
	lhs.z /= rhs.z;
	lhs.w /= rhs.w;
	return lhs;
}

inline vec5 make_vec5(Float scalar) {
	vec5 result;
	result.x = scalar;
	result.y = scalar;
	result.z = scalar;
	result.w = scalar;
	result.a = scalar;
	return result;
}
inline vec5 make_vec5(Float x, vec4 yzwa) {
	vec5 result;
	result.x = x;
	result.yzwa = yzwa;
	return result;
}
inline vec5 make_vec5(vec2 xy, vec3 zwa) {
	vec5 result;
	result.xy = xy;
	result.zwa = zwa;
	return result;
}
inline vec5 make_vec5(vec3 xyz, vec2 wa) {
	vec5 result;
	result.xyz = xyz;
	result.wa = wa;
	return result;
}
inline vec5 make_vec5(vec4 xyzw, Float a) {
	vec5 result;
	result.xyzw = xyzw;
	result.a = a;
	return result;
}
inline vec5 make_vec5(Float x, Float y, vec3 zwa) {
	vec5 result;
	result.x = x;
	result.y = y;
	result.zwa = zwa;
	return result;
}
inline vec5 make_vec5(Float x, vec2 yz, vec2 wa) {
	vec5 result;
	result.x = x;
	result.yz = yz;
	result.wa = wa;
	return result;
}
inline vec5 make_vec5(Float x, vec3 yzw, Float a) {
	vec5 result;
	result.x = x;
	result.yzw = yzw;
	result.a = a;
	return result;
}
inline vec5 make_vec5(vec2 xy, Float z, vec2 wa) {
	vec5 result;
	result.xy = xy;
	result.z = z;
	result.wa = wa;
	return result;
}
inline vec5 make_vec5(vec2 xy, vec2 zw, Float a) {
	vec5 result;
	result.xy = xy;
	result.zw = zw;
	result.a = a;
	return result;
}
inline vec5 make_vec5(vec3 xyz, Float w, Float a) {
	vec5 result;
	result.xyz = xyz;
	result.w = w;
	result.a = a;
	return result;
}
inline vec5 make_vec5(Float x, Float y, Float z, vec2 wa) {
	vec5 result;
	result.x = x;
	result.y = y;
	result.z = z;
	result.wa = wa;
	return result;
}
inline vec5 make_vec5(Float x, Float y, vec2 zw, Float a) {
	vec5 result;
	result.x = x;
	result.y = y;
	result.zw = zw;
	result.a = a;
	return result;
}
inline vec5 make_vec5(Float x, vec2 yz, Float w, Float a) {
	vec5 result;
	result.x = x;
	result.yz = yz;
	result.w = w;
	result.a = a;
	return result;
}
inline vec5 make_vec5(vec2 xy, Float z, Float w, Float a) {
	vec5 result;
	result.xy = xy;
	result.z = z;
	result.w = w;
	result.a = a;
	return result;
}
inline vec5 make_vec5(Float x, Float y, Float z, Float w, Float a) {
	vec5 result;
	result.x = x;
	result.y = y;
	result.z = z;
	result.w = w;
	result.a = a;
	return result;
}

inline vec5 operator-(const vec5& rhs) {
	vec5 result;
	result.x = -rhs.x;
	result.y = -rhs.y;
	result.z = -rhs.z;
	result.w = -rhs.w;
	result.a = -rhs.a;
	return result;
}

inline vec5 operator+(Float lhs, const vec5& rhs) {
	vec5 result;
	result.x = lhs + rhs.x;
	result.y = lhs + rhs.y;
	result.z = lhs + rhs.z;
	result.w = lhs + rhs.w;
	result.a = lhs + rhs.a;
	return result;
}

inline vec5 operator+(const vec5& lhs, Float rhs) {
	vec5 result;
	result.x = lhs.x + rhs;
	result.y = lhs.y + rhs;
	result.z = lhs.z + rhs;
	result.w = lhs.w + rhs;
	result.a = lhs.a + rhs;
	return result;
}

inline vec5 operator+(const vec5& lhs, const vec5& rhs) {
	vec5 result;
	result.x = lhs.x + rhs.x;
	result.y = lhs.y + rhs.y;
	result.z = lhs.z + rhs.z;
	result.w = lhs.w + rhs.w;
	result.a = lhs.a + rhs.a;
	return result;
}

inline const vec5& operator+=(vec5& lhs, Float rhs) {
	lhs.x += rhs;
	lhs.y += rhs;
	lhs.z += rhs;
	lhs.w += rhs;
	lhs.a += rhs;
	return lhs;
}

inline const vec5& operator+=(vec5& lhs, const vec5& rhs) {
	lhs.x += rhs.x;
	lhs.y += rhs.y;
	lhs.z += rhs.z;
	lhs.w += rhs.w;
	lhs.a += rhs.a;
	return lhs;
}

inline vec5 operator-(Float lhs, const vec5& rhs) {
	vec5 result;
	result.x = lhs - rhs.x;
	result.y = lhs - rhs.y;
	result.z = lhs - rhs.z;
	result.w = lhs - rhs.w;
	result.a = lhs - rhs.a;
	return result;
}

inline vec5 operator-(const vec5& lhs, Float rhs) {
	vec5 result;
	result.x = lhs.x - rhs;
	result.y = lhs.y - rhs;
	result.z = lhs.z - rhs;
	result.w = lhs.w - rhs;
	result.a = lhs.a - rhs;
	return result;
}

inline vec5 operator-(const vec5& lhs, const vec5& rhs) {
	vec5 result;
	result.x = lhs.x - rhs.x;
	result.y = lhs.y - rhs.y;
	result.z = lhs.z - rhs.z;
	result.w = lhs.w - rhs.w;
	result.a = lhs.a - rhs.a;
	return result;
}

inline const vec5& operator-=(vec5& lhs, Float rhs) {
	lhs.x -= rhs;
	lhs.y -= rhs;
	lhs.z -= rhs;
	lhs.w -= rhs;
	lhs.a -= rhs;
	return lhs;
}

inline const vec5& operator-=(vec5& lhs, const vec5& rhs) {
	lhs.x -= rhs.x;
	lhs.y -= rhs.y;
	lhs.z -= rhs.z;
	lhs.w -= rhs.w;
	lhs.a -= rhs.a;
	return lhs;
}

inline vec5 operator*(Float lhs, const vec5& rhs) {
	vec5 result;
	result.x = lhs * rhs.x;
	result.y = lhs * rhs.y;
	result.z = lhs * rhs.z;
	result.w = lhs * rhs.w;
	result.a = lhs * rhs.a;
	return result;
}

inline vec5 operator*(const vec5& lhs, Float rhs) {
	vec5 result;
	result.x = lhs.x * rhs;
	result.y = lhs.y * rhs;
	result.z = lhs.z * rhs;
	result.w = lhs.w * rhs;
	result.a = lhs.a * rhs;
	return result;
}

inline vec5 operator*(const vec5& lhs, const vec5& rhs) {
	vec5 result;
	result.x = lhs.x * rhs.x;
	result.y = lhs.y * rhs.y;
	result.z = lhs.z * rhs.z;
	result.w = lhs.w * rhs.w;
	result.a = lhs.a * rhs.a;
	return result;
}

inline const vec5& operator*=(vec5& lhs, Float rhs) {
	lhs.x *= rhs;
	lhs.y *= rhs;
	lhs.z *= rhs;
	lhs.w *= rhs;
	lhs.a *= rhs;
	return lhs;
}

inline const vec5& operator*=(vec5& lhs, const vec5& rhs) {
	lhs.x *= rhs.x;
	lhs.y *= rhs.y;
	lhs.z *= rhs.z;
	lhs.w *= rhs.w;
	lhs.a *= rhs.a;
	return lhs;
}

inline vec5 operator/(Float lhs, const vec5& rhs) {
	vec5 result;
	result.x = lhs / rhs.x;
	result.y = lhs / rhs.y;
	result.z = lhs / rhs.z;
	result.w = lhs / rhs.w;
	result.a = lhs / rhs.a;
	return result;
}

inline vec5 operator/(const vec5& lhs, Float rhs) {
	vec5 result;
	result.x = lhs.x / rhs;
	result.y = lhs.y / rhs;
	result.z = lhs.z / rhs;
	result.w = lhs.w / rhs;
	result.a = lhs.a / rhs;
	return result;
}

inline vec5 operator/(const vec5& lhs, const vec5& rhs) {
	vec5 result;
	result.x = lhs.x / rhs.x;
	result.y = lhs.y / rhs.y;
	result.z = lhs.z / rhs.z;
	result.w = lhs.w / rhs.w;
	result.a = lhs.a / rhs.a;
	return result;
}

inline const vec5& operator/=(vec5& lhs, Float rhs) {
	lhs.x /= rhs;
	lhs.y /= rhs;
	lhs.z /= rhs;
	lhs.w /= rhs;
	lhs.a /= rhs;
	return lhs;
}

inline const vec5& operator/=(vec5& lhs, const vec5& rhs) {
	lhs.x /= rhs.x;
	lhs.y /= rhs.y;
	lhs.z /= rhs.z;
	lhs.w /= rhs.w;
	lhs.a /= rhs.a;
	return lhs;
}

inline vec6 make_vec6(Float scalar) {
	vec6 result;
	result.x = scalar;
	result.y = scalar;
	result.z = scalar;
	result.w = scalar;
	result.a = scalar;
	result.b = scalar;
	return result;
}
inline vec6 make_vec6(Float x, vec5 yzwab) {
	vec6 result;
	result.x = x;
	result.yzwab = yzwab;
	return result;
}
inline vec6 make_vec6(vec2 xy, vec4 zwab) {
	vec6 result;
	result.xy = xy;
	result.zwab = zwab;
	return result;
}
inline vec6 make_vec6(vec3 xyz, vec3 wab) {
	vec6 result;
	result.xyz = xyz;
	result.wab = wab;
	return result;
}
inline vec6 make_vec6(vec4 xyzw, vec2 ab) {
	vec6 result;
	result.xyzw = xyzw;
	result.ab = ab;
	return result;
}
inline vec6 make_vec6(vec5 xyzwa, Float b) {
	vec6 result;
	result.xyzwa = xyzwa;
	result.b = b;
	return result;
}
inline vec6 make_vec6(Float x, Float y, vec4 zwab) {
	vec6 result;
	result.x = x;
	result.y = y;
	result.zwab = zwab;
	return result;
}
inline vec6 make_vec6(Float x, vec2 yz, vec3 wab) {
	vec6 result;
	result.x = x;
	result.yz = yz;
	result.wab = wab;
	return result;
}
inline vec6 make_vec6(Float x, vec3 yzw, vec2 ab) {
	vec6 result;
	result.x = x;
	result.yzw = yzw;
	result.ab = ab;
	return result;
}
inline vec6 make_vec6(Float x, vec4 yzwa, Float b) {
	vec6 result;
	result.x = x;
	result.yzwa = yzwa;
	result.b = b;
	return result;
}
inline vec6 make_vec6(vec2 xy, Float z, vec3 wab) {
	vec6 result;
	result.xy = xy;
	result.z = z;
	result.wab = wab;
	return result;
}
inline vec6 make_vec6(vec2 xy, vec2 zw, vec2 ab) {
	vec6 result;
	result.xy = xy;
	result.zw = zw;
	result.ab = ab;
	return result;
}
inline vec6 make_vec6(vec2 xy, vec3 zwa, Float b) {
	vec6 result;
	result.xy = xy;
	result.zwa = zwa;
	result.b = b;
	return result;
}
inline vec6 make_vec6(vec3 xyz, Float w, vec2 ab) {
	vec6 result;
	result.xyz = xyz;
	result.w = w;
	result.ab = ab;
	return result;
}
inline vec6 make_vec6(vec3 xyz, vec2 wa, Float b) {
	vec6 result;
	result.xyz = xyz;
	result.wa = wa;
	result.b = b;
	return result;
}
inline vec6 make_vec6(vec4 xyzw, Float a, Float b) {
	vec6 result;
	result.xyzw = xyzw;
	result.a = a;
	result.b = b;
	return result;
}
inline vec6 make_vec6(Float x, Float y, Float z, vec3 wab) {
	vec6 result;
	result.x = x;
	result.y = y;
	result.z = z;
	result.wab = wab;
	return result;
}
inline vec6 make_vec6(Float x, Float y, vec2 zw, vec2 ab) {
	vec6 result;
	result.x = x;
	result.y = y;
	result.zw = zw;
	result.ab = ab;
	return result;
}
inline vec6 make_vec6(Float x, Float y, vec3 zwa, Float b) {
	vec6 result;
	result.x = x;
	result.y = y;
	result.zwa = zwa;
	result.b = b;
	return result;
}
inline vec6 make_vec6(Float x, vec2 yz, Float w, vec2 ab) {
	vec6 result;
	result.x = x;
	result.yz = yz;
	result.w = w;
	result.ab = ab;
	return result;
}
inline vec6 make_vec6(Float x, vec2 yz, vec2 wa, Float b) {
	vec6 result;
	result.x = x;
	result.yz = yz;
	result.wa = wa;
	result.b = b;
	return result;
}
inline vec6 make_vec6(Float x, vec3 yzw, Float a, Float b) {
	vec6 result;
	result.x = x;
	result.yzw = yzw;
	result.a = a;
	result.b = b;
	return result;
}
inline vec6 make_vec6(vec2 xy, Float z, Float w, vec2 ab) {
	vec6 result;
	result.xy = xy;
	result.z = z;
	result.w = w;
	result.ab = ab;
	return result;
}
inline vec6 make_vec6(vec2 xy, Float z, vec2 wa, Float b) {
	vec6 result;
	result.xy = xy;
	result.z = z;
	result.wa = wa;
	result.b = b;
	return result;
}
inline vec6 make_vec6(vec2 xy, vec2 zw, Float a, Float b) {
	vec6 result;
	result.xy = xy;
	result.zw = zw;
	result.a = a;
	result.b = b;
	return result;
}
inline vec6 make_vec6(vec3 xyz, Float w, Float a, Float b) {
	vec6 result;
	result.xyz = xyz;
	result.w = w;
	result.a = a;
	result.b = b;
	return result;
}
inline vec6 make_vec6(Float x, Float y, Float z, Float w, vec2 ab) {
	vec6 result;
	result.x = x;
	result.y = y;
	result.z = z;
	result.w = w;
	result.ab = ab;
	return result;
}
inline vec6 make_vec6(Float x, Float y, Float z, vec2 wa, Float b) {
	vec6 result;
	result.x = x;
	result.y = y;
	result.z = z;
	result.wa = wa;
	result.b = b;
	return result;
}
inline vec6 make_vec6(Float x, Float y, vec2 zw, Float a, Float b) {
	vec6 result;
	result.x = x;
	result.y = y;
	result.zw = zw;
	result.a = a;
	result.b = b;
	return result;
}
inline vec6 make_vec6(Float x, vec2 yz, Float w, Float a, Float b) {
	vec6 result;
	result.x = x;
	result.yz = yz;
	result.w = w;
	result.a = a;
	result.b = b;
	return result;
}
inline vec6 make_vec6(vec2 xy, Float z, Float w, Float a, Float b) {
	vec6 result;
	result.xy = xy;
	result.z = z;
	result.w = w;
	result.a = a;
	result.b = b;
	return result;
}
inline vec6 make_vec6(Float x, Float y, Float z, Float w, Float a, Float b) {
	vec6 result;
	result.x = x;
	result.y = y;
	result.z = z;
	result.w = w;
	result.a = a;
	result.b = b;
	return result;
}

inline vec6 operator-(const vec6& rhs) {
	vec6 result;
	result.x = -rhs.x;
	result.y = -rhs.y;
	result.z = -rhs.z;
	result.w = -rhs.w;
	result.a = -rhs.a;
	result.b = -rhs.b;
	return result;
}

inline vec6 operator+(Float lhs, const vec6& rhs) {
	vec6 result;
	result.x = lhs + rhs.x;
	result.y = lhs + rhs.y;
	result.z = lhs + rhs.z;
	result.w = lhs + rhs.w;
	result.a = lhs + rhs.a;
	result.b = lhs + rhs.b;
	return result;
}

inline vec6 operator+(const vec6& lhs, Float rhs) {
	vec6 result;
	result.x = lhs.x + rhs;
	result.y = lhs.y + rhs;
	result.z = lhs.z + rhs;
	result.w = lhs.w + rhs;
	result.a = lhs.a + rhs;
	result.b = lhs.b + rhs;
	return result;
}

inline vec6 operator+(const vec6& lhs, const vec6& rhs) {
	vec6 result;
	result.x = lhs.x + rhs.x;
	result.y = lhs.y + rhs.y;
	result.z = lhs.z + rhs.z;
	result.w = lhs.w + rhs.w;
	result.a = lhs.a + rhs.a;
	result.b = lhs.b + rhs.b;
	return result;
}

inline const vec6& operator+=(vec6& lhs, Float rhs) {
	lhs.x += rhs;
	lhs.y += rhs;
	lhs.z += rhs;
	lhs.w += rhs;
	lhs.a += rhs;
	lhs.b += rhs;
	return lhs;
}

inline const vec6& operator+=(vec6& lhs, const vec6& rhs) {
	lhs.x += rhs.x;
	lhs.y += rhs.y;
	lhs.z += rhs.z;
	lhs.w += rhs.w;
	lhs.a += rhs.a;
	lhs.b += rhs.b;
	return lhs;
}

inline vec6 operator-(Float lhs, const vec6& rhs) {
	vec6 result;
	result.x = lhs - rhs.x;
	result.y = lhs - rhs.y;
	result.z = lhs - rhs.z;
	result.w = lhs - rhs.w;
	result.a = lhs - rhs.a;
	result.b = lhs - rhs.b;
	return result;
}

inline vec6 operator-(const vec6& lhs, Float rhs) {
	vec6 result;
	result.x = lhs.x - rhs;
	result.y = lhs.y - rhs;
	result.z = lhs.z - rhs;
	result.w = lhs.w - rhs;
	result.a = lhs.a - rhs;
	result.b = lhs.b - rhs;
	return result;
}

inline vec6 operator-(const vec6& lhs, const vec6& rhs) {
	vec6 result;
	result.x = lhs.x - rhs.x;
	result.y = lhs.y - rhs.y;
	result.z = lhs.z - rhs.z;
	result.w = lhs.w - rhs.w;
	result.a = lhs.a - rhs.a;
	result.b = lhs.b - rhs.b;
	return result;
}

inline const vec6& operator-=(vec6& lhs, Float rhs) {
	lhs.x -= rhs;
	lhs.y -= rhs;
	lhs.z -= rhs;
	lhs.w -= rhs;
	lhs.a -= rhs;
	lhs.b -= rhs;
	return lhs;
}

inline const vec6& operator-=(vec6& lhs, const vec6& rhs) {
	lhs.x -= rhs.x;
	lhs.y -= rhs.y;
	lhs.z -= rhs.z;
	lhs.w -= rhs.w;
	lhs.a -= rhs.a;
	lhs.b -= rhs.b;
	return lhs;
}

inline vec6 operator*(Float lhs, const vec6& rhs) {
	vec6 result;
	result.x = lhs * rhs.x;
	result.y = lhs * rhs.y;
	result.z = lhs * rhs.z;
	result.w = lhs * rhs.w;
	result.a = lhs * rhs.a;
	result.b = lhs * rhs.b;
	return result;
}

inline vec6 operator*(const vec6& lhs, Float rhs) {
	vec6 result;
	result.x = lhs.x * rhs;
	result.y = lhs.y * rhs;
	result.z = lhs.z * rhs;
	result.w = lhs.w * rhs;
	result.a = lhs.a * rhs;
	result.b = lhs.b * rhs;
	return result;
}

inline vec6 operator*(const vec6& lhs, const vec6& rhs) {
	vec6 result;
	result.x = lhs.x * rhs.x;
	result.y = lhs.y * rhs.y;
	result.z = lhs.z * rhs.z;
	result.w = lhs.w * rhs.w;
	result.a = lhs.a * rhs.a;
	result.b = lhs.b * rhs.b;
	return result;
}

inline const vec6& operator*=(vec6& lhs, Float rhs) {
	lhs.x *= rhs;
	lhs.y *= rhs;
	lhs.z *= rhs;
	lhs.w *= rhs;
	lhs.a *= rhs;
	lhs.b *= rhs;
	return lhs;
}

inline const vec6& operator*=(vec6& lhs, const vec6& rhs) {
	lhs.x *= rhs.x;
	lhs.y *= rhs.y;
	lhs.z *= rhs.z;
	lhs.w *= rhs.w;
	lhs.a *= rhs.a;
	lhs.b *= rhs.b;
	return lhs;
}

inline vec6 operator/(Float lhs, const vec6& rhs) {
	vec6 result;
	result.x = lhs / rhs.x;
	result.y = lhs / rhs.y;
	result.z = lhs / rhs.z;
	result.w = lhs / rhs.w;
	result.a = lhs / rhs.a;
	result.b = lhs / rhs.b;
	return result;
}

inline vec6 operator/(const vec6& lhs, Float rhs) {
	vec6 result;
	result.x = lhs.x / rhs;
	result.y = lhs.y / rhs;
	result.z = lhs.z / rhs;
	result.w = lhs.w / rhs;
	result.a = lhs.a / rhs;
	result.b = lhs.b / rhs;
	return result;
}

inline vec6 operator/(const vec6& lhs, const vec6& rhs) {
	vec6 result;
	result.x = lhs.x / rhs.x;
	result.y = lhs.y / rhs.y;
	result.z = lhs.z / rhs.z;
	result.w = lhs.w / rhs.w;
	result.a = lhs.a / rhs.a;
	result.b = lhs.b / rhs.b;
	return result;
}

inline const vec6& operator/=(vec6& lhs, Float rhs) {
	lhs.x /= rhs;
	lhs.y /= rhs;
	lhs.z /= rhs;
	lhs.w /= rhs;
	lhs.a /= rhs;
	lhs.b /= rhs;
	return lhs;
}

inline const vec6& operator/=(vec6& lhs, const vec6& rhs) {
	lhs.x /= rhs.x;
	lhs.y /= rhs.y;
	lhs.z /= rhs.z;
	lhs.w /= rhs.w;
	lhs.a /= rhs.a;
	lhs.b /= rhs.b;
	return lhs;
}

inline mat2 operator-(const mat2& rhs) {
	mat2 result;
	result.columns[0] = -rhs[0];
	result.columns[1] = -rhs[1];
	return result;
}

inline mat2 operator+(Float lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs + rhs.columns[0];
	result.columns[1] = lhs + rhs.columns[1];
	return result;
}

inline mat2 operator+(const mat2& lhs, Float rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] + rhs;
	result.columns[1] = lhs.columns[1] + rhs;
	return result;
}

inline mat2 operator+(const mat2& lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] + rhs.columns[0];
	result.columns[1] = lhs.columns[1] + rhs.columns[1];
	return result;
}

inline const mat2& operator+=(mat2& lhs, Float rhs) {
	lhs.columns[0] += rhs;
	lhs.columns[1] += rhs;
	return lhs;
}

inline const mat2& operator+=(mat2& lhs, const mat2& rhs) {
	lhs.columns[0] += rhs.columns[0];
	lhs.columns[1] += rhs.columns[1];
	return lhs;
}

inline mat2 operator-(Float lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs - rhs.columns[0];
	result.columns[1] = lhs - rhs.columns[1];
	return result;
}

inline mat2 operator-(const mat2& lhs, Float rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] - rhs;
	result.columns[1] = lhs.columns[1] - rhs;
	return result;
}

inline mat2 operator-(const mat2& lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] - rhs.columns[0];
	result.columns[1] = lhs.columns[1] - rhs.columns[1];
	return result;
}

inline const mat2& operator-=(mat2& lhs, Float rhs) {
	lhs.columns[0] -= rhs;
	lhs.columns[1] -= rhs;
	return lhs;
}

inline const mat2& operator-=(mat2& lhs, const mat2& rhs) {
	lhs.columns[0] -= rhs.columns[0];
	lhs.columns[1] -= rhs.columns[1];
	return lhs;
}

inline mat2 operator*(Float lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs * rhs.columns[0];
	result.columns[1] = lhs * rhs.columns[1];
	return result;
}

inline mat2 operator*(const mat2& lhs, Float rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] * rhs;
	result.columns[1] = lhs.columns[1] * rhs;
	return result;
}

inline mat2 operator*(const mat2& lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] * rhs.columns[0];
	result.columns[1] = lhs.columns[1] * rhs.columns[1];
	return result;
}

inline const mat2& operator*=(mat2& lhs, Float rhs) {
	lhs.columns[0] *= rhs;
	lhs.columns[1] *= rhs;
	return lhs;
}

inline const mat2& operator*=(mat2& lhs, const mat2& rhs) {
	lhs.columns[0] *= rhs.columns[0];
	lhs.columns[1] *= rhs.columns[1];
	return lhs;
}

inline mat2 operator/(Float lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs / rhs.columns[0];
	result.columns[1] = lhs / rhs.columns[1];
	return result;
}

inline mat2 operator/(const mat2& lhs, Float rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] / rhs;
	result.columns[1] = lhs.columns[1] / rhs;
	return result;
}

inline mat2 operator/(const mat2& lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] / rhs.columns[0];
	result.columns[1] = lhs.columns[1] / rhs.columns[1];
	return result;
}

inline const mat2& operator/=(mat2& lhs, Float rhs) {
	lhs.columns[0] /= rhs;
	lhs.columns[1] /= rhs;
	return lhs;
}

inline const mat2& operator/=(mat2& lhs, const mat2& rhs) {
	lhs.columns[0] /= rhs.columns[0];
	lhs.columns[1] /= rhs.columns[1];
	return lhs;
}

inline vec2 operator*(const mat2& lhs, const vec2& rhs) {
	vec2 result;
	result[0] = lhs[0][0] * rhs[0] + lhs[1][0] * rhs[1];
	result[1] = lhs[0][1] * rhs[0] + lhs[1][1] * rhs[1];
	return result;
}

inline mat2 transpose(const mat2& rhs) {
	mat2 result;
	result.columns[0][0] = rhs.columns[0][0];
	result.columns[1][0] = rhs.columns[0][1];
	result.columns[0][1] = rhs.columns[1][0];
	result.columns[1][1] = rhs.columns[1][1];
	return result;
}

inline mat3 operator-(const mat3& rhs) {
	mat3 result;
	result.columns[0] = -rhs[0];
	result.columns[1] = -rhs[1];
	result.columns[2] = -rhs[2];
	return result;
}

inline mat3 operator+(Float lhs, const mat3& rhs) {
	mat3 result;
	result.columns[0] = lhs + rhs.columns[0];
	result.columns[1] = lhs + rhs.columns[1];
	result.columns[2] = lhs + rhs.columns[2];
	return result;
}

inline mat3 operator+(const mat3& lhs, Float rhs) {
	mat3 result;
	result.columns[0] = lhs.columns[0] + rhs;
	result.columns[1] = lhs.columns[1] + rhs;
	result.columns[2] = lhs.columns[2] + rhs;
	return result;
}

inline mat3 operator+(const mat3& lhs, const mat3& rhs) {
	mat3 result;
	result.columns[0] = lhs.columns[0] + rhs.columns[0];
	result.columns[1] = lhs.columns[1] + rhs.columns[1];
	result.columns[2] = lhs.columns[2] + rhs.columns[2];
	return result;
}

inline const mat3& operator+=(mat3& lhs, Float rhs) {
	lhs.columns[0] += rhs;
	lhs.columns[1] += rhs;
	lhs.columns[2] += rhs;
	return lhs;
}

inline const mat3& operator+=(mat3& lhs, const mat3& rhs) {
	lhs.columns[0] += rhs.columns[0];
	lhs.columns[1] += rhs.columns[1];
	lhs.columns[2] += rhs.columns[2];
	return lhs;
}

inline mat3 operator-(Float lhs, const mat3& rhs) {
	mat3 result;
	result.columns[0] = lhs - rhs.columns[0];
	result.columns[1] = lhs - rhs.columns[1];
	result.columns[2] = lhs - rhs.columns[2];
	return result;
}

inline mat3 operator-(const mat3& lhs, Float rhs) {
	mat3 result;
	result.columns[0] = lhs.columns[0] - rhs;
	result.columns[1] = lhs.columns[1] - rhs;
	result.columns[2] = lhs.columns[2] - rhs;
	return result;
}

inline mat3 operator-(const mat3& lhs, const mat3& rhs) {
	mat3 result;
	result.columns[0] = lhs.columns[0] - rhs.columns[0];
	result.columns[1] = lhs.columns[1] - rhs.columns[1];
	result.columns[2] = lhs.columns[2] - rhs.columns[2];
	return result;
}

inline const mat3& operator-=(mat3& lhs, Float rhs) {
	lhs.columns[0] -= rhs;
	lhs.columns[1] -= rhs;
	lhs.columns[2] -= rhs;
	return lhs;
}

inline const mat3& operator-=(mat3& lhs, const mat3& rhs) {
	lhs.columns[0] -= rhs.columns[0];
	lhs.columns[1] -= rhs.columns[1];
	lhs.columns[2] -= rhs.columns[2];
	return lhs;
}

inline mat3 operator*(Float lhs, const mat3& rhs) {
	mat3 result;
	result.columns[0] = lhs * rhs.columns[0];
	result.columns[1] = lhs * rhs.columns[1];
	result.columns[2] = lhs * rhs.columns[2];
	return result;
}

inline mat3 operator*(const mat3& lhs, Float rhs) {
	mat3 result;
	result.columns[0] = lhs.columns[0] * rhs;
	result.columns[1] = lhs.columns[1] * rhs;
	result.columns[2] = lhs.columns[2] * rhs;
	return result;
}

inline mat3 operator*(const mat3& lhs, const mat3& rhs) {
	mat3 result;
	result.columns[0] = lhs.columns[0] * rhs.columns[0];
	result.columns[1] = lhs.columns[1] * rhs.columns[1];
	result.columns[2] = lhs.columns[2] * rhs.columns[2];
	return result;
}

inline const mat3& operator*=(mat3& lhs, Float rhs) {
	lhs.columns[0] *= rhs;
	lhs.columns[1] *= rhs;
	lhs.columns[2] *= rhs;
	return lhs;
}

inline const mat3& operator*=(mat3& lhs, const mat3& rhs) {
	lhs.columns[0] *= rhs.columns[0];
	lhs.columns[1] *= rhs.columns[1];
	lhs.columns[2] *= rhs.columns[2];
	return lhs;
}

inline mat3 operator/(Float lhs, const mat3& rhs) {
	mat3 result;
	result.columns[0] = lhs / rhs.columns[0];
	result.columns[1] = lhs / rhs.columns[1];
	result.columns[2] = lhs / rhs.columns[2];
	return result;
}

inline mat3 operator/(const mat3& lhs, Float rhs) {
	mat3 result;
	result.columns[0] = lhs.columns[0] / rhs;
	result.columns[1] = lhs.columns[1] / rhs;
	result.columns[2] = lhs.columns[2] / rhs;
	return result;
}

inline mat3 operator/(const mat3& lhs, const mat3& rhs) {
	mat3 result;
	result.columns[0] = lhs.columns[0] / rhs.columns[0];
	result.columns[1] = lhs.columns[1] / rhs.columns[1];
	result.columns[2] = lhs.columns[2] / rhs.columns[2];
	return result;
}

inline const mat3& operator/=(mat3& lhs, Float rhs) {
	lhs.columns[0] /= rhs;
	lhs.columns[1] /= rhs;
	lhs.columns[2] /= rhs;
	return lhs;
}

inline const mat3& operator/=(mat3& lhs, const mat3& rhs) {
	lhs.columns[0] /= rhs.columns[0];
	lhs.columns[1] /= rhs.columns[1];
	lhs.columns[2] /= rhs.columns[2];
	return lhs;
}

inline vec3 operator*(const mat3& lhs, const vec3& rhs) {
	vec3 result;
	result[0] = lhs[0][0] * rhs[0] + lhs[1][0] * rhs[1] + lhs[2][0] * rhs[2];
	result[1] = lhs[0][1] * rhs[0] + lhs[1][1] * rhs[1] + lhs[2][1] * rhs[2];
	result[2] = lhs[0][2] * rhs[0] + lhs[1][2] * rhs[1] + lhs[2][2] * rhs[2];
	return result;
}

inline mat3 transpose(const mat3& rhs) {
	mat3 result;
	result.columns[0][0] = rhs.columns[0][0];
	result.columns[1][0] = rhs.columns[0][1];
	result.columns[2][0] = rhs.columns[0][2];
	result.columns[0][1] = rhs.columns[1][0];
	result.columns[1][1] = rhs.columns[1][1];
	result.columns[2][1] = rhs.columns[1][2];
	result.columns[0][2] = rhs.columns[2][0];
	result.columns[1][2] = rhs.columns[2][1];
	result.columns[2][2] = rhs.columns[2][2];
	return result;
}

