#  Copyright (C) 2021, Christoph Peters, Karlsruhe Institute of Technology
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.


from numpy import dot, maximum, minimum, sqrt, sin, cos, abs, arcsin, arccos, arctan, arctan2, clip, cross, pi, zeros, \
    asarray, linspace, where, concatenate, outer, linalg, transpose, eye, copy, float32, uint32, isinf, isnan, inf


def inversesqrt(x):
    return 1.0 / sqrt(x)


def fma(a, b, c):
    return a * b + c


def saturate(x):
    return clip(x, 0.0, 1.0)


def mix(x, y, s):
    return (1.0 - s) * x + s * y


def normalize(vector):
    return inversesqrt(dot(vector, vector)) * vector


def vec(broadcast_size, *arguments):
    if len(arguments) == 1 or len(asarray(arguments[0]).shape) == 0:
        result = asarray(arguments)
    else:
        result = concatenate([asarray(argument) for argument in arguments])
    if result.size == 1:
        result = result.repeat(broadcast_size)
    return result


def vec2(*arguments):
    return vec(2, *arguments)


def vec3(*arguments):
    return vec(3, *arguments)


def vec4(*arguments):
    return vec(4, *arguments)

def mat2(*arguments):
    if len(arguments) == 0:
        return asarray([[1.0, 0.0], [0.0, 1.0]])
    elif len(arguments) == 2:
        return asarray(arguments).T
    elif len(arguments) == 4:
        return asarray([[arguments[0], arguments[2]], [arguments[1], arguments[3]]])


def length(rhs):
    return linalg.norm(rhs)


def floatBitsToUint(rhs):
    return copy(asarray(rhs, dtype=float32).view(uint32))

def uintBitsToFloat(rhs):
    return copy(asarray(rhs, dtype=uint32).view(float32))


def where(condition, yes, no):
    return yes if condition else no